<?php

/**
 * @file
 * Map field values with static map.
 */

/**
 * Search API data alteration callback.
 */
class SearchApiMapFieldValues extends SearchApiAbstractAlterCallback {

  /**
   * Overrides SearchApiAbstractAlterCallback::supportsIndex().
   *
   * Returns TRUE only for indexes on nodes.
   */
  public function supportsIndex(SearchApiIndex $index) {
    return TRUE;
  }

  /**
   * Alter the items.
   */
  public function alterItems(array &$items) {
    if (!$items) {
      return;
    }
    
    if (isset($this->options['fields'])) {
      foreach ($items as $key => $item) {
        $wrapper = $this->index->entityWrapper($item);
        $vals = array();

        foreach ($this->options['fields'] as $name => $field) {
          

          if ($field['name']) {
            $map = array();
            $mapping_lines = explode(PHP_EOL, trim($field['mapping']));
            foreach ($mapping_lines as $line) {
              $key_value = explode('|', $line);
              $map[trim($key_value[0])] = !empty($key_value[1]) ? t(trim($key_value[1])) : '';
            }

            $required_field[$field['base_field']]['type'] = 'integer';
            $r_field = search_api_extract_fields($wrapper, $required_field);

            if (!isset($map[$r_field[$field['base_field']]['value']])) {
              $items[$key]->{$name} = $r_field[$field['base_field']]['value'];
              continue;
            }
            
            $items[$key]->{$name} = $map[$r_field[$field['base_field']]['value']];
          }
        }
      }
    }
  }

  /**
   * Overrides SearchApiAbstractAlterCallback::propertyInfo().
   *
   * Adds the "search_api_field_mapping" properties.
   */
  public function propertyInfo() {
    $ret = array();
    if (isset($this->options['fields'])) {
      foreach ($this->options['fields'] as $name => $field) {
        $type = (TRUE === isset($field['type'])) ? $field['type'] : 'integer';

        $ret[$name] = array(
          'label' => $field['name'],
          'description' => empty($field['description']) ? '' : $field['description'],
          'type' => $type,
        );
      }
    }

    return $ret;
  }

  public function configurationForm() {
    $form['#attached']['css'][] = drupal_get_path('module', 'search_api') . '/search_api.admin.css';

    $fields = $this->index->getFields(FALSE);
    $field_options = array();

    foreach ($fields as $name => $field) {
      $field_options[$name] = t('@label (@name)', array('@label' => $field['name'], '@name' => $name));
    }

    $facets = array('' => t('None')) + $field_options;
    
    $additional = empty($this->options['fields']) ? array() : $this->options['fields'];

    $form['#id'] = 'edit-callbacks-search-api-alter-add-mapping-settings';
    $form['description'] = array(
      '#markup' => t('<p>This data alteration lets you define additional fields that will be added to this index.</p>')
          . t('<p>Whether you need the source key or the source value for the mapping depends on the field type.</p>'),
    );

    $form['fields']['#prefix'] = '<div id="search-api-alter-add-mapping-field-settings">';
    $form['fields']['#suffix'] = '</div>';
    if (isset($this->changes)) {
      $form['fields']['#prefix'] .= '<div class="messages warning">All changes in the form will not be saved until the <em>Save configuration</em> button at the form bottom is clicked.</div>';
    }

    foreach ($additional as $name => $field) {
      $form['fields'][$name] = array(
        '#type' => 'fieldset',
        '#title' => $field['name'] ? $field['name'] : t('New field'),
        '#collapsible' => TRUE,
        '#collapsed' => (boolean) $field['name'],
      );

      $form['fields'][$name]['name'] = array(
        '#type' => 'textfield',
        '#title' => t('New field name'),
        '#default_value' => $field['name'],
        '#required' => TRUE,
      );

      $form['fields'][$name]['base_field'] = array(
        '#type' => 'select',
        '#title' => t('Select field for mapping'),
        '#options' => $facets,
        '#default_value' => $field['base_field'],
        '#required' => TRUE,
        '#description' => t('Treat this field as another type of field, useful for making a mapping taxonomy field behave as such for FacetAPI')
      );

      $form['fields'][$name]['type'] = array(
      	'#type' => 'select',
        '#title' => t('Target data type'),
        '#options' => search_api_default_field_types(),
        '#default_value' => (TRUE === isset($field['type'])) ? $field['type'] : 'integer',
        '#required' => TRUE,
        '#description' => t('Data type to save field as'),
      );

      $form['fields'][$name]['mapping'] = array(
      	'#type' => 'textarea',
        '#title' => t('Field value mapping'),
        '#default_value' => $field['mapping'],
        '#required' => TRUE,
        '#description' => t('Use "orignal_value|new_value". Only one mapping per line.'),
      );

      $form['fields'][$name]['actions'] = array(
        '#type' => 'actions',
        'remove' => array(
          '#type' => 'submit',
          '#value' => t('Remove field'),
          '#submit' => array('_search_api_add_mapping_field_submit'),
          '#limit_validation_errors' => array(),
          '#name' => 'search_api_add_mapping_remove_' . $name,
          '#ajax' => array(
            'callback' => '_search_api_add_mapping_field_ajax',
            'wrapper' => 'search-api-alter-add-mapping-field-settings',
          ),
        ),
      );
    }
    $form['actions']['#type'] = 'actions';
    $form['actions']['add_field_mapping'] = array(
      '#type' => 'submit',
      '#value' => t('Add new mapping field'),
      '#submit' => array('_search_api_add_mapping_field_submit'),
      '#limit_validation_errors' => array(),
      '#ajax' => array(
        'callback' => '_search_api_add_mapping_field_ajax',
        'wrapper' => 'search-api-alter-add-mapping-field-settings',
      ),
    );
    return $form;
  }

  public function configurationFormSubmit(array $form, array &$values, array &$form_state) {
    if (empty($values['fields'])) {
      return array();
    }

    $index_fields = $this->index->getFields(FALSE);
    foreach ($values['fields'] as $name => $field) {
      if (!$field['name']) {
        unset($values['fields'][$name]);
      }
      else {
        $values['fields'][$name]['description'] = $this->fieldDescription($field, $index_fields);
      }
    }
    $this->options = $values;
    return $values;
  }

  /**
   * Submit helper callback for buttons in the callback's configuration form.
   */
  public function formButtonSubmit(array $form, array &$form_state) {
    $button_name = $form_state['triggering_element']['#name'];
    if ($button_name == 'op') {
      for ($i = 1; isset($this->options['fields']['search_api_mapping_' . $i]); ++$i) {
      }
      $this->options['fields']['search_api_mapping_' . $i] = array(
        'name' => '',
        'type' => 'fulltext',
        'fields' => array(),
      );
    }
    else {
      $field = substr($button_name, 31);
      unset($this->options['fields'][$field]);
    }
    $form_state['rebuild'] = TRUE;
    $this->changes = TRUE;
  }

  /**
   * Helper method for creating a field description.
   */
  protected function fieldDescription(array $field, array $index_fields) {
    return t('A static mapping of the following field: @field.', array('@field' => $field['base_field']));
  }

}

/**
 * Submit function for buttons in the callback's configuration form.
 */
function _search_api_add_mapping_field_submit(array $form, array &$form_state) {
  $form_state['callbacks']['search_api_alter_add_mapping']->formButtonSubmit($form, $form_state);
}

/**
 * AJAX submit function for buttons in the callback's configuration form.
 */
function _search_api_add_mapping_field_ajax(array $form, array &$form_state) {
  return $form['callbacks']['settings']['search_api_alter_add_mapping']['fields'];
}
